package iTunesBackup;

import javax.swing.*;
import javax.swing.filechooser.*;
import java.awt.*;
import java.awt.event.*;
import java.util.*;
import java.net.*;
import java.io.*;
/**
 * Tune Backup
 * (c) 2007 Gabe Albert
 * Gabe Albert - gabeosx (at) yahoo.com - www.gabealbert.info
 * The use and distribution of this work is subject to the terms of the 
 * GNU GPL license.  A full-text copy of the license is available online at
 * http://www.gnu.org/licenses/gpl.txt
 * @author Gabe Albert
 */
public class TuneController implements ActionListener {
	private JProgressBar progBar = new JProgressBar();
	private JLabel statusLabel = new JLabel();
	private BackerUpper bu;
	private JButton syncButton = new JButton("Sync");
	private JTextField xmlLocation = new JTextField(20);
	private JButton browseXMLButton = new JButton("Browse");
	private JButton browseFolderButton = new JButton("Browse");
	private JTextField destinationLocation = new JTextField(20);
	private JLabel destinationLabel = new JLabel();
	private DBPipe dbPipe = new DBPipe();
	HashMap<Integer, Thread> threads;
	private static String defaultLocale = System.getProperty("user.home")+"/Music/iTunes/iTunes Music Library.xml";
	
	private String xmlFileLocation;
	private String destinationFolderLocation ="";
	
	private TrackCollection badTracks = new TrackCollection();
	
	public TuneController() {
		//dbPipe.testDBPipe();
		syncButton.setEnabled(true);
		syncButton.addActionListener(this);
		dbPipe.startConnect();
		xmlFileLocation=dbPipe.getPref("xmlpath");
		if(xmlFileLocation.equalsIgnoreCase("") || xmlFileLocation==null)
			xmlFileLocation=defaultLocale;
		xmlLocation.setText(xmlFileLocation);
		destinationFolderLocation = dbPipe.getPref("backupfolder");
		destinationLocation.setText(dbPipe.getPref("backupfolder"));

		createGui();
	}
	
	public TrackCollection getBadTracks() {
		return badTracks;
	}

	public void createGui() {
		browseXMLButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent ae) {
				browseForFile(xmlLocation, new ChooserFileFilter("xml"), defaultLocale);
				xmlFileLocation = xmlLocation.getText();
			}
		});
		browseFolderButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent ae) {
				browseForFolder(destinationLocation);
				destinationFolderLocation = destinationLocation.getText();
			}
		});

		JFrame.setDefaultLookAndFeelDecorated(true);
		JFrame frame = new JFrame("Tune Backup");
		JPanel xmlFilePanel = new JPanel();
		JPanel syncPanel = new JPanel();
		JPanel destinationPanel = new JPanel();
		JPanel infoPanel = new JPanel();
		infoPanel.setLayout(new GridLayout(2, 0));
		syncPanel.setLayout(new FlowLayout(FlowLayout.CENTER));
		syncPanel.add(syncButton);
		syncPanel.add(statusLabel);
		xmlFilePanel.setLayout(new FlowLayout());
		xmlFilePanel.add(new JLabel("        iTunes Library "));
		xmlFilePanel.add(xmlLocation);
		xmlFilePanel.add(browseXMLButton);
		destinationPanel.setLayout(new FlowLayout());
		destinationPanel.add(new JLabel("Backup Destination "));
		destinationPanel.add(destinationLocation);
		destinationPanel.add(browseFolderButton);
		frame.setLayout(new GridLayout(5, 0));
		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		infoPanel.add(new JLabel("(c) 2007 Gabe Albert", JLabel.CENTER));
		infoPanel.add(new JLabel("www.gabealbert.info/tunebackup", JLabel.CENTER));
		statusLabel.setText("            Welcome to Tune Backup            ");
		
		syncButton.addActionListener(this);	
		frame.getContentPane().add(syncPanel);
		frame.getContentPane().add(infoPanel);
		frame.getContentPane().add(xmlFilePanel);
		frame.getContentPane().add(destinationPanel);
		frame.getContentPane().add(progBar);
		frame.pack();
		xmlLocation.setText(defaultLocale);
		frame.setVisible(true);
	}

	public void actionPerformed(ActionEvent e) {
		if(e.getSource().equals(syncButton)) {
			syncButton.setEnabled(false);
			syncButton.setSelected(false);
			new Thread (bu).start();
			syncButton.setEnabled(true);
		}
		if(e.getSource().equals(syncButton)) {
			if(xmlFileLocation!=null && !xmlFileLocation.equalsIgnoreCase("") && !destinationFolderLocation.equalsIgnoreCase("") && destinationFolderLocation!=null) {
//				statusLabel.setText("Syncing... this may take a while");
				syncButton.setEnabled(false);
				dbPipe.setPref("xmlpath", xmlFileLocation);
				dbPipe.setPref("backupfolder", destinationFolderLocation);
				syncAction();
			}
			else {
				statusLabel.setText("Please verify library & destination are set");
			}
		}
	}
	
	public int getFirstDeadThread() {
		int thread=-1;
		while(thread==-1) {
			for(int i=0; i<threads.size(); i++) {
				if(!threads.get(i).isAlive()) {
					thread=i;
					return i;
				}
			}
		}
		return thread;
	}

	public void setProgressMax(int max) {
		progBar.setMaximum(max);
	}

	public void setProgressCurrent(int current) {
		progBar.setValue(current);
		progBar.setString(current+"/"+progBar.getMaximum());
		if(progBar.getValue()==progBar.getMaximum()) {
			statusLabel.setText("Done");
			syncButton.setEnabled(true);
			postProcessHook();
		}
	}
	
	private void postProcessHook() {
		if(badTracks!=null && badTracks.size()>0) {
			new ErrorViewerController(badTracks);
			badTracks = null;
		}
	}

	public void setProgressString(String theString) {
		statusLabel.setText(theString);
	}

	public static void main(String args[]) {
		TuneController tc = new TuneController();
	}
	
	public void browseForFile(JTextField textField, ChooserFileFilter fileFilter, String defaultLocation) {
		JFileChooser fc = new JFileChooser(defaultLocation);
		fc.addChoosableFileFilter(fileFilter);
		fc.showOpenDialog(null);
		textField.setText(fc.getSelectedFile().getAbsolutePath());
	}
	
	public void browseForFolder(JTextField textField) {
		JFileChooser fc = new JFileChooser();
		fc.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY );
		fc.showOpenDialog(null);
		textField.setText(fc.getSelectedFile().getAbsolutePath());
	}
	
	public void syncAction() {
//		BackerUpper bu = new BackerUpper(this, xmlFileLocation); // initialize track info
		Thread threadMaster = new BackerUpper(BackerUpper.MANAGE_THREADS, xmlFileLocation, destinationFolderLocation, this);
		threadMaster.start();
	}
}
